
__version__ = "0.1.0"

import importlib
import json
import logging
import os
import time

from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, Optional, Set, Union

import paho.mqtt.client as mqtt

from tomlkit import parse


MI_TEMPERATURE = "temperature"
MI_LIGHT = "light"
MI_MOISTURE = "moisture"
MI_CONDUCTIVITY = "conductivity"
MI_BATTERY = "battery"

DEVICE_PREFIX = "miblepy_"
DEFAULT_MAX_RETRIES = 1

DEFAULT_CONFIG_FILE = f"{os.environ.get('HOME', '')}/.miblepy.toml"


class ATTRS(Enum):
    """Attributes sent in the json dict."""

    AGE = "age"
    BASAL_METABOLISM = "basal_metabolism"
    BATTERY = "battery"
    BMI = "bmi"
    BODY_FAT = "body_fat"
    BONE_MASS = "bone_mass"
    BRIGHTNESS = "brightness"
    CONDUCTIVITY = "conductivity"
    HEIGHT = "height"
    HUMIDITY = "humidity"
    IMPEDANCE = "impedance"
    LEAN_BODY_MASS = "lean_body_mass"
    MOISTURE = "moisture"
    MUSCLE_MASS = "muscle_mass"
    MQTT_SUFFIX = "suffix"
    PROTEIN = "protein"
    SEX = "sex"
    TEMPERATURE = "temperature"
    TIMESTAMP = "timestamp"
    UNIT = "unit"
    USER = "user"
    VISCERAL_FAT = "visceral_fat"
    VOLTAGE = "voltage"
    WATER = "water"
    WEIGHT = "weight"


# unit of measurement for the different attributes
UNIT_OF_MEASUREMENT = {
    ATTRS.BATTERY: "%",
    ATTRS.VOLTAGE: "V",
    ATTRS.TEMPERATURE: "°C",
    ATTRS.BRIGHTNESS: "lux",
    ATTRS.MOISTURE: "%",
    ATTRS.HUMIDITY: "%",
    ATTRS.IMPEDANCE: "Ω",
    ATTRS.CONDUCTIVITY: "µS/cm",
    ATTRS.TIMESTAMP: "s",
    ATTRS.HEIGHT: "cm",
    ATTRS.WEIGHT: "kg",
    ATTRS.UNIT: None,
    ATTRS.USER: None,
}


# home assistant device classes for the different attributes
DEVICE_CLASS = {
    ATTRS.BATTERY: "battery",
    ATTRS.VOLTAGE: "battery",
    ATTRS.TEMPERATURE: "temperature",
    ATTRS.BRIGHTNESS: "illuminance",
    ATTRS.MOISTURE: None,
    ATTRS.HUMIDITY: None,
    ATTRS.CONDUCTIVITY: None,
    ATTRS.TIMESTAMP: "timestamp",
    ATTRS.WEIGHT: None,
    ATTRS.IMPEDANCE: None,
    ATTRS.UNIT: None,
    ATTRS.USER: None,
}


def hl(text: Union[int, float, str]) -> str:
    return f"\033[1m{text}\033[0m"


# pylint: disable-msg=too-many-instance-attributes
class Configuration:
    """Stores the program configuration."""

    def __init__(self, config_file_path: str, debug: bool = False):
        with open(config_file_path, "r") as file:
            config_file = parse(file.read())

        self.config_file = config_file
        self.config = {}

        config_general = config_file.get("general")

        # logging
        if debug:
            config_general["debug"] = debug
        self.debug: str = config_general.get("debug", False)
        self._configure_logging(config_general)

        # ble interface
        self.interface: str = config_general.get("interface", "hci0")
        self.max_retries: str = config_general.get("max_retries", DEFAULT_MAX_RETRIES)

        #  mqtt
        mqtt_settings: Dict[str, Any] = {}
        config_mqtt = config_file.get("mqtt")
        if "server" not in config_mqtt:
            logging.error("no mqtt server")
        else:
            mqtt_settings["server"]: str = config_mqtt.get("server")
            mqtt_settings["port"]: int = config_mqtt.get("port", 8883)
            mqtt_settings["client_id"]: Optional[str] = config_mqtt.get("client_id")
            mqtt_settings["user"]: Optional[str] = config_mqtt.get("username")
            mqtt_settings["password"]: Optional[str] = config_mqtt.get("password")
            mqtt_settings["discovery_prefix"]: Optional[str] = config_mqtt.get("discovery_prefix")
            mqtt_settings["prefix"] = config_mqtt.get("prefix", "miblepy/")
            mqtt_settings["trailing_slash"]: bool = config_mqtt.get("trailing_slash", False)
            mqtt_settings["timestamp_format"]: Optional[str] = config_mqtt.get("timestamp_format")
            mqtt_settings["ca_cert"]: Optional[str] = config_mqtt.get("ca_cert")

        # sensors
        if "sensors" not in config_file:
            logging.error("no mqtt server")
        else:
            sensors: List[DeviceConfig] = []

            for device_type in config_file.get("sensors", {}):
                for sensor in config_file["sensors"][device_type]:
                    fail_silent = "fail_silent" in sensor
                    sensors.append(
                        DeviceConfig(sensor["mac"], sensor["alias"], device_type, fail_silent)
                    )

        self.sensors = sensors
        self.mqtt = mqtt_settings

        def __str__(self) -> str:
            return self.config_file

    @staticmethod
    def _configure_logging(config):
        timeform = "%Y-%m-%d %H:%M:%S"
        logform = "%(asctime)s %(levelname)-7s %(message)s"
        loglevel = logging.INFO if not config["debug"] else logging.DEBUG

        if "logfile" in config:
            logfile = os.path.abspath(os.path.expanduser(config["logfile"]))
            logging.basicConfig(filename=logfile, level=loglevel, datefmt=timeform, format=logform)
        else:
            logging.basicConfig(level=loglevel, datefmt=timeform, format=logform)


class DeviceConfig:
    """Stores the configuration of a sensor."""

    def __init__(
        self, mac: str, alias: str = None, device_type: str = None, fail_silent: bool = False
    ):
        if not mac:
            logging.exception("mac of sensor must not be None")

        self.mac = mac
        self.alias = alias
        self.device_type = device_type
        self.fail_silent = fail_silent

    @property
    def name(self) -> str:
        return self.alias if self.alias else self.mac

    @property
    def short_mac(self) -> str:
        """Get the sensor mac without ':' in it."""
        return self.mac.replace(":", "")

    def get_topic(self) -> str:
        """Get the topic name for the sensor."""
        return f"{self.alias.replace(' ', '_')}_{self.short_mac}" if self.alias else self.mac

    def __str__(self) -> str:
        return f"{self.alias if self.alias else self.mac}{' (fail silent)' if self.fail_silent else ''}"

    @staticmethod
    def get_name_string(sensor_list: List) -> str:
        """Convert a list of sensor objects to a nice string."""
        return ", ".join((str(sensor.alias) for sensor in sensor_list))


class Miblepy:
    """Main class of the module."""

    def __init__(self, config_file_path: str = "~/.miblepy.toml", retries: int = DEFAULT_MAX_RETRIES, debug: bool = False):
        config_file_path = os.path.abspath(os.path.expanduser(config_file_path))
        self.config = Configuration(config_file_path, debug=debug)

        logging.info("")
        logging.info(f"{hl(__name__)} {__version__}")
        logging.info("")
        logging.info(f"config file: {hl(config_file_path)}")
        logging.info(f"  interface: /dev/{hl(self.config.interface)}")
        logging.info(f"      debug: {hl(self.config.debug)}")
        logging.debug("")
        logging.debug(f"configuration: {self.config.config_file}")
        logging.info("")

        self.mqtt_client = None
        self.connected = False

        self.config.max_retries = retries

        # sys.exit(0)

    def start_client(self):
        """Start the mqtt client."""
        if not self.connected:
            self._start_client()

    def stop_client(self):
        """Stop the mqtt client."""
        if self.connected:
            self.mqtt_client.disconnect()
            self.connected = False
        self.mqtt_client.loop_stop()
        logging.debug("")
        logging.debug(
            f"disconnected MQTT connection to server {hl(self.config.mqtt['server'] + ':' + str(self.config.mqtt['port']))}"
        )

    def _start_client(self):
        self.mqtt_client: mqtt.Client = mqtt.Client(self.config.mqtt["client_id"])

        if self.config.mqtt["user"]:
            self.mqtt_client.username_pw_set(self.config.mqtt["user"], self.config.mqtt["password"])

        if self.config.mqtt["ca_cert"]:
            self.mqtt_client.tls_set(self.config.mqtt["ca_cert"], cert_reqs=mqtt.ssl.CERT_REQUIRED)

        def _on_connect(client, _, flags, return_code):
            self.connected = True
            logging.info(
                f"MQTT connection to {hl(self.config.mqtt['server'] + ':' + str(self.config.mqtt['port']))} established"  #: {mqtt.connack_string(return_code)}"
            )

        self.mqtt_client.on_connect = _on_connect

        logging.debug(
            f"MQTT connecting to {hl(self.config.mqtt['server'] + ':' + str(self.config.mqtt['port']))}..."
        )
        self.mqtt_client.connect(str(self.config.mqtt["server"]), int(self.config.mqtt["port"]), 60)
        self.mqtt_client.loop_start()

    def _publisher(self, topic: str, data: str):
        # state_topic = state_topic.replace(" ", "_")

        if self.config.mqtt["timestamp_format"]:
            data["timestamp"] = datetime.now().strftime(self.config.mqtt["timestamp_format"])

        msg: mqtt.MQTTMessageInfo = self.mqtt_client.publish(
            topic, json.dumps(data), qos=1, retain=True
        )
        msg.wait_for_publish()
        logging.debug(f"sent {data} to topic {topic} - published: {msg.is_published()}")

    @staticmethod
    def _get_device_topic(sensor_config: DeviceConfig, suffix: Optional[str] = None) -> str:
        device_topic = (
            f"{sensor_config.short_mac}_{sensor_config.alias.replace(' ', '_')}"
            if sensor_config.alias
            else sensor_config.short_mac
        )

        return f"{device_topic}{f'_{suffix}' if suffix else ''}"

    def _get_state_topic(self, sensor_config: DeviceConfig, suffix: Optional[str] = None) -> str:
        """Construct state topic to publish to."""
        device_topic = self._get_device_topic(sensor_config, suffix)

        return f"{self.config.mqtt['prefix']}/{device_topic}{'/' if self.config.mqtt['trailing_slash'] else ''}"

    def _get_announce_topic(self, sensor_config: str, attribute: str, suffix: Optional[str] = None) -> str:
        """Construct announce topic to publish to."""
        device_topic = self._get_device_topic(sensor_config, suffix)

        return f"{self.config.mqtt['discovery_prefix']}/sensor/{device_topic}_{attribute}/config"

    def fetch(self, sensor_config: DeviceConfig) -> Dict[str, Any]:
        """Get data from one Sensor."""
        logging.info("")
        logging.info(f"fetching data from sensor {hl(sensor_config.name)} ({sensor_config.mac})...")

        try:
            device_backend = importlib.import_module(f"miblepy.devices.{sensor_config.device_type}")
        except (ModuleNotFoundError, ImportError) as error:
            logging.exception(f"required module {sensor_config.device_type} not found: {error}")
            return

        data = device_backend.fetch_data(sensor_config.mac, self.config.interface)

        if not data:
            logging.warning(f"  no data received from backend {device_backend.__name__} for device {hl(sensor_config.name)} ({sensor_config.mac})!")
            return None

        mqtt_device_suffix = data.get(ATTRS.MQTT_SUFFIX.value, None)
        state_topic = self._get_state_topic(sensor_config, mqtt_device_suffix)

        self.announce_sensor(
            device_backend.SUPPORTED_ATTRS,
            sensor_config,
            state_topic,
            suffix=mqtt_device_suffix
        )
        logging.info(
            f"  sent sensor configuration to {hl(self._get_announce_topic(sensor_config, '<attribute>', suffix=mqtt_device_suffix))}"
        )

        # push sensor values
        self._publisher(state_topic, data)
        logging.info(f"  sent sensor values to {hl(state_topic)}")

        return data

    def go(self):
        """Get data from all sensors."""
        sensors_list: Set[DeviceConfig] = set(self.config.sensors)

        # initial timeout in seconds
        timeout = 1

        # number of retries
        max_retries = self.config.max_retries

        retry_count = 1

        while not self.connected:
            self.start_client()
            time.sleep(0.1)

        while retry_count <= max_retries and sensors_list:

            # if this is not the first try: wait some time before trying again
            if retry_count > 1:
                logging.info("")
                logging.info(f"try {retry_count}/{max_retries} for {', '.join((hl(str(sensor.alias)) for sensor in sensors_list))} in {timeout}s")
                time.sleep(timeout)

                # exponential backoff-time
                timeout *= 2

            # collect failed sensor for next round
            failed_sensors_list: Set[DeviceConfig] = set()

            # increment retry counter
            retry_count += 1

            # process sensors in list
            for sensor in sensors_list:

                try:
                    if not self.fetch(sensor):
                        # try again in the next round
                        failed_sensors_list.add(sensor)

                except Exception as exception:  # pylint: disable=bare-except, broad-except

                    # try again in the next round
                    failed_sensors_list.add(sensor)

                    msg = "could not read data from {} ({}) with reason: {}".format(
                        sensor.mac, sensor.alias, str(exception)
                    )

                    if sensor.fail_silent:
                        logging.error(msg)
                        logging.warning(
                            "fail_silent is set for sensor %s, so not raising an exception.",
                            sensor.alias,
                        )
                    else:
                        logging.exception(msg)
                        print(msg)

            sensors_list = failed_sensors_list

        logging.info("")

        # return sensors that could not be processed after max_retries
        return failed_sensors_list

    def announce_sensor(
        self, supported_attributes: List[ATTRS], sensor_config: DeviceConfig, state_topic: str, suffix: Optional[str] = None
    ) -> None:
        """Announce the sensor via Home Assistant MQTT Discovery: https://www.home-assistant.io/docs/mqtt/discovery/"""

        for attribute in supported_attributes:

            payload = {
                "state_topic": state_topic,
                "unit_of_measurement": UNIT_OF_MEASUREMENT[attribute],
                "value_template": "{{value_json." + attribute.value + "}}",
            }
            if sensor_config.alias:
                payload["name"] = f"{sensor_config.alias} {attribute.value}"

            if DEVICE_CLASS[attribute]:
                payload["device_class"] = DEVICE_CLASS[attribute]

            announce_topic = self._get_announce_topic(sensor_config, attribute.value, suffix=suffix)

            # payloads.append(payload)
            self._publisher(announce_topic, payload)




# supported devices
#   Mi Body Composition Scale 2 (XMTZC05HM)

import logging
import time

from datetime import date, datetime
from typing import Any, Dict, List, Union

import miblepy.devices.xbm as xbm

from bluepy import btle

from .. import ATTRS


SUPPORTED_ATTRS = [
    ATTRS.WEIGHT,
    ATTRS.IMPEDANCE,
    ATTRS.UNIT,
    # ATTRS.USER,
    ATTRS.TIMESTAMP,
]


MISCALE_USERS: List[Dict[str, Union[int, float, str, date]]] = [
    dict(
        user="Ben",
        height=181,
        birthdate=date(1989, 6, 15),
        sex="male",
        weightOver=60,
        weightBelow=80,
    ),
    dict(
        user="Syra",
        height=159,
        birthdate=date(1990, 2, 21),
        sex="female",
        weightOver=40,
        weightBelow=60,
    ),
    dict(
        user="Shiva",
        height=42,
        birthdate=date(2011, 4, 11),
        sex="female",
        weightOver=3,
        weightBelow=20,
    ),
]


sensor_data = {}

SCAN_TIMEOUT = 10


def fetch_data(mac: str, interface: str) -> Dict[str, Any]:
    """Get data from one Sensor."""

    try:
        btle.Scanner().withDelegate(ScanProcessor(mac)).scan(SCAN_TIMEOUT)
    except btle.BTLEDisconnectError as error:
        logging.error(f"btle disconnected: {error}")
    except btle.BTLEManagementError as error:
        logging.error(f"(temporary) bluetooth connection error: {error}")

    if not sensor_data:
        time.sleep(1)

    logging.debug(f"{sensor_data = }")

    return sensor_data


def find_user(weight: float) -> str:

    def get_age(birthdate: Any) -> int:
        today = date.today()
        return int(today.year - birthdate.year - ((today.month, today.day) < (birthdate.month, birthdate.day)))

    current_user: Dict[str, Union[int, float, str, date]] = {}

    # determine current user by weight
    for user in MISCALE_USERS:

        if user["weightOver"] and not user["weightBelow"]:
            if weight > user["weightOver"]:
                current_user = user

        elif user["weightOver"] and user["weightBelow"]:
            if weight > user["weightOver"] and weight < user["weightBelow"]:
                current_user = user

        elif user["weightOver"] and user["weightBelow"]:
            if weight > user["weightOver"] and weight < user["weightBelow"]:
                current_user = user

        # if current user found, fill profile values and exit
        if current_user:
            current_user[ATTRS.WEIGHT.value] = weight
            current_user[ATTRS.AGE.value] = get_age(current_user["birthdate"])
            break

    return current_user


class ScanProcessor:
    def __init__(self, mac):
        self.mac = mac
        self.mqtt_client = None
        self.connected = False
        btle.DefaultDelegate.__init__(self)

    def swapi16(self, data: bytes, start_byte: int, end_byte: int) -> int:
        return int((data[end_byte - 2 : end_byte] + data[start_byte : start_byte + 2]), 16)

    def handleDiscovery(self, dev, new_dev, new_data):
        """Handle discovered devices"""
        global sensor_data

        if dev.addr == self.mac.lower() and new_dev:

            for (sdid, desc, data) in dev.getScanData():

                dt = None
                impedance = None
                measurement_weight = None
                unit = None

                # Mi Body Composition Scale 2 (XMTZC05HM) / Xiaomi Scale 2 (XMTZC02HM)
                if data.startswith("1b18") and sdid == 22:

                    # parse weight & unit
                    measurement_weight = self.swapi16(data, 26, 30) * 0.01
                    measurement_unit = data[4:6]

                    # check weight unit
                    if measurement_unit == "03":
                        unit = "lbs"
                    if measurement_unit == "02":
                        unit = "kg"
                        measurement_weight = measurement_weight / 2
                    else:
                        logging.error(f"No known unit found! Got: {measurement_unit}")
                        return

                    # parse received bytes
                    impedance = int((data[24:26] + data[22:24]), 16)
                    measurement_date = f"{self.swapi16(data, 8, 12)}-{int((data[12:14]), 16)}-{int((data[14:16]), 16)}"
                    measurement_time = f"{int((data[16:18]), 16)}:{int((data[18:20]), 16)}:{int((data[20:22]), 16)}"
                    dt = datetime.strptime(
                        f"{measurement_date} {measurement_time}", "%Y-%m-%d %H:%M:%S"
                    )

                    logging.debug(f"{measurement_weight = }")
                    logging.debug(f"{impedance = }")
                    logging.debug(f"{dt = }")

                    # fake data
                    measurement_weight = 8.05
                    impedance = 1500

                    if (current_user := find_user(measurement_weight)):
                        bm = xbm.bodyMetrics(
                            current_user[ATTRS.WEIGHT.value],
                            current_user[ATTRS.HEIGHT.value],
                            current_user[ATTRS.AGE.value],
                            current_user[ATTRS.SEX.value],
                            impedance,
                        )

                        # check if we got everything we need
                        if not all([dt, impedance, (impedance > 0 and impedance < 3000), measurement_weight, unit]):
                            logging.warning(
                                f"missing data! measurement_weight: {measurement_weight} | unit: {unit} | dt: {dt} | impedance: {impedance}"
                            )
                            return

                        sensor_data = {
                            ATTRS.USER.value: current_user[ATTRS.USER.value],
                            ATTRS.AGE.value: current_user[ATTRS.AGE.value],
                            ATTRS.SEX.value: current_user[ATTRS.SEX.value],
                            ATTRS.HEIGHT.value: current_user[ATTRS.HEIGHT.value],
                            ATTRS.WEIGHT.value: round(measurement_weight, 2),
                            ATTRS.UNIT.value: unit,
                            ATTRS.IMPEDANCE.value: impedance,
                            ATTRS.BASAL_METABOLISM.value: f"{bm.get_bmr():.2f}",
                            ATTRS.VISCERAL_FAT.value: f"{bm.getVisceralFat():.2f}",
                            ATTRS.BMI.value: f"{bm.getBMI():.2f}",
                            ATTRS.WATER.value: f"{bm.getWaterPercentage():.2f}",
                            ATTRS.BONE_MASS.value: f"{bm.getBoneMass():.2f}",
                            ATTRS.BODY_FAT.value: f"{bm.getFatPercentage():.2f}",
                            ATTRS.LEAN_BODY_MASS.value: f"{bm.get_lbm_coefficient():.2f}",
                            ATTRS.MUSCLE_MASS.value: f"{bm.getMuscleMass():.2f}",
                            ATTRS.PROTEIN.value: f"{bm.getProteinPercentage():.2f}",
                            ATTRS.TIMESTAMP.value: dt.timestamp(),
                            ATTRS.MQTT_SUFFIX.value: current_user[ATTRS.USER.value],
                        }




# supported devices
#   VegTrug Plant Sensor (???)
#   Mi Flora? (???)

from datetime import datetime
from typing import Any, Dict

from btlewrap.bluepy import BluepyBackend
from miflora.miflora_poller import MiFloraPoller

from miblepy import ATTRS, MI_BATTERY, MI_CONDUCTIVITY, MI_LIGHT, MI_MOISTURE, MI_TEMPERATURE


SUPPORTED_ATTRS = [ATTRS.VOLTAGE, ATTRS.TEMPERATURE, ATTRS.MOISTURE, ATTRS.CONDUCTIVITY, ATTRS.TIMESTAMP]


def fetch_data(mac: str, interface: str) -> Dict[str, Any]:
    """Get data from one Sensor."""

    poller = MiFloraPoller(mac, BluepyBackend, adapter=interface, retries=1)

    data = {
        ATTRS.BATTERY.value: poller.parameter_value(MI_BATTERY),
        ATTRS.TEMPERATURE.value: "{0:.1f}".format(poller.parameter_value(MI_TEMPERATURE)),
        ATTRS.BRIGHTNESS.value: poller.parameter_value(MI_LIGHT),
        ATTRS.MOISTURE.value: poller.parameter_value(MI_MOISTURE),
        ATTRS.CONDUCTIVITY.value: poller.parameter_value(MI_CONDUCTIVITY),
        ATTRS.TIMESTAMP.value: datetime.now().isoformat(),
    }

    return data




# supported devices
#   Mijia LCD Temperature Humidity Sensor (LYWSD03MMC)

import logging

from datetime import datetime
from typing import Any, Dict

from bluepy import btle

from miblepy import ATTRS


SUPPORTED_ATTRS = [ATTRS.VOLTAGE, ATTRS.TEMPERATURE, ATTRS.HUMIDITY, ATTRS.TIMESTAMP]


def fetch_data(mac: str, interface: str) -> Dict[str, Any]:
    """Get data from one Sensor."""

    sensor_data = {}

    try:
        peripheral = connect(mac, interface, sensor_data)

        if peripheral.waitForNotifications(2000):
            peripheral.disconnect()
    except btle.BTLEDisconnectError as error:
        logging.error(f"btle disconnected: {error}")
    except Exception as error:
        logging.exception(f"error when trying to fetch data from {mac}: {error}")

    return sensor_data


class MiblepyDelegate(btle.DefaultDelegate):

    def __init__(self, sensor_data):
        self.sensor_data: Dict[str, Any] = sensor_data
        btle.DefaultDelegate.__init__(self)

    def handleNotification(self, cHandle: int, data: bytes):
        # global sensor_data

        if cHandle == 54:

            try:
                self.sensor_data.update({
                    ATTRS.VOLTAGE.value: str(int.from_bytes(data[3:5], byteorder="little") / 1000.0),
                    ATTRS.TEMPERATURE.value: str(int.from_bytes(data[0:2], byteorder="little", signed=True) / 100),
                    ATTRS.HUMIDITY.value: str(int.from_bytes(data[2:3], byteorder="little")),
                    ATTRS.TIMESTAMP.value: str(datetime.now().isoformat()),
                })

            except (TypeError, ValueError) as error:
                logging.error(f"parsing sensor data failed: {error}")
                logging.error(f"sensor data: {data}")


def connect(mac: str, interface: str, sensor_data: Dict[str, Any]):
    """Connect to device and activate notifications."""
    interface_idx: int = int(interface.replace("hci", ""))

    peripheral = btle.Peripheral(mac, iface=interface_idx)

    # enable temperature, humidity and battery voltage notifications
    peripheral.writeCharacteristic(0x38, bytes([0x01, 0x00]), True)
    peripheral.writeCharacteristic(0x46, bytes([0xf4, 0x01, 0x00]), True)

    # register handler
    peripheral.withDelegate(MiblepyDelegate(sensor_data))

    return peripheral



#!/usr/bin/env python3

import click

from miblepy import (
    DEFAULT_CONFIG_FILE,
    DEFAULT_MAX_RETRIES,
    Miblepy,
    __name__ as miblepy_name,
    __version__ as miblepy_version,
    hl,
)


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.command(context_settings=CONTEXT_SETTINGS)
@click.option("-c", "--config", default=DEFAULT_CONFIG_FILE, type=click.Path(exists=True), help="path to config file")
@click.option(
    "-r",
    "--retries",
    default=DEFAULT_MAX_RETRIES,
    type=click.IntRange(0, 10, clamp=True),
    help="times we try to get data from a sensor",
)
@click.option("--version", default=False, is_flag=True, help=f"show {__name__} version")
@click.option("-d", "--debug", default=False, is_flag=True, help="enable debug output")
def cli(config: str, retries: int, version: bool, debug: bool):
    """fetch data from ble sensors and push it to a mqtt broker

      https://github.com/benleb/miblepy
    """

    if version:
        click.echo(f"{miblepy_name} {hl(miblepy_version)} | https://github.com/benleb/miblepy | @benleb")
        exit(0)

    Miblepy(config_file_path=config, retries=retries, debug=debug).go()


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['miblepy', 'miblepy.devices', 'miblepy.devices.xbm']

package_data = \
{'': ['*']}

install_requires = \
['bluepy>=1.3,<2.0',
 'btlewrap>=0.0.8,<0.0.9',
 'click>=7.1.2,<8.0.0',
 'miflora>=0.6,<0.7',
 'paho-mqtt>=1.5.0,<2.0.0',
 'tomlkit>=0.6.0,<0.7.0']

entry_points = \
{'console_scripts': ['miblepy = miblepy.cli:cli']}

setup_kwargs = {
    'name': 'miblepy',
    'version': '0.1.0',
    'description': 'miblepy fetches data from various (Xiaomi/Mijia/Mi) Bluetooth LE devices and push it to a MQTT broker in a coordinated, sequential manner.',
    'long_description': None,
    'author': 'Ben Lebherz',
    'author_email': 'git@benleb.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
